@@ -485,19 +485,26 @@ class CurveBuilder:
             orientation = [Orientation.DOWN_RIGHT, Orientation.RIGHT_DOWN]
         return orientation
 
-    def curves_from_points(self, curve_type: CurveType, points: List[Vector2]) -> List[Curve]:
+    def curves_from_points(self, curve_type: CurveType, points: List[Vector2]) -> (List[Curve], List[Vector2]):
         curves = []
+        leftover_points = []
         if len(points) <2:
-            return curves
+            return curves, leftover_points
+
 
         if curve_type == CurveType.LINE:
             curves = self.lines_from_points(points)
         elif curve_type == CurveType.ARC:
             curves = self.arcs_from_points(points)
         elif curve_type == CurveType.CUBICBEZIER:
-            curves = self.cubicbeziers_from_points(points)
+            leftover_count = (len(self.current_points) - 1) % 3 if len(self.current_points) >= 4 else len(self.current_points)
+            leftover_points = points[-leftover_count:] if leftover_count != 0 else leftover_points
+            used_points = points[:-leftover_count] if leftover_count != 0 else points
+            curves = self.cubicbeziers_from_points(used_points)
+        elif curve_type == CurveType.CATMULLROM:
+            curves = self.catmullrom_from_points(points)
 
-        return curves
+        return curves, leftover_points
 
     def lines_from_points(self, points):
         lines = []
@@ -519,27 +526,72 @@ class CurveBuilder:
 
     def cubicbeziers_from_points(self, points):
         curves = []
+
+        if len(points) < 4:
+            return curves
+
         # A Cubic Bezier requires 4 control points, so operate in blocks of 4 (re-using the last each time)
-        for ii in range(0, len(points) - 3, 3):
-            curves.append(CubicBezier(self.id_provider(), points[ii:ii+4]))
+        # for ii in range(0, len(points) - 3, 3):
+        #     curves.append(CubicBezier(self.id_provider(), points[ii:ii+4]))
+
+        curves.append(CubicBezier(self.id_provider(), points))
         return curves
 
-    def curves_from_temp_next_points(self, next_points: List[Vector2], force_continuity: bool=True):
+    def catmullrom_from_points(self, points):
+        curves = []
+
+        curves.append(CatmullRom(self.id_provider(), points))
+        return curves
+
+    def curves_from_temp_next_points(self, next_points: List[Vector2]):
         all_points = self.current_points + next_points
         points = []
         for i in all_points:
             if i not in points:
                 points.append(i)
 
-        return self.curves_from_points(self.current_curve_type.value, points)
+        curves, leftovers = self.curves_from_points(self.current_curve_type.value, points)
+
+        return curves
 
     def force_continuity(self, next_point: Vector2):
-        curves = self.curves_from_points(self.current_curve_type.value, self.current_points)
+        curves, leftovers = self.curves_from_points(self.current_curve_type.value, self.current_points)
 
         if curves is None or len(curves) == 0:
             return next_point
 
-        return curves[-1].force_continuity(next_point)
+        # Dont need to force a continuity if we are in the middle of a Bezier
+        if self.current_curve_type.value == CurveType.CUBICBEZIER and len(leftovers) != 0:
+            return next_point
+        else:
+            return curves[-1].force_continuity(next_point)
+
+
+if __name__ == "__main__":
+    # builder = CurveBuilder(lambda: "1")
+    #
+    # a = Vector2(1, 2)
+    # b = Vector2(1, 3)
+    # c = Vector2(1, 4)
+    # d = Vector2(1, 2)
+    # e = Vector2(1, 1)
+    #
+    # builder.set_curve_type(CurveType.CUBICBEZIER)
+    #
+    # builder.add_point(a)
+    # builder.add_point(b)
+    # builder.add_point(c)
+    # builder.add_point(d)
+    # builder.add_point(e)
+    #
+    # not_continuous = Vector2(100.5, 100.5)
+    # continuous = builder.force_continuity(not_continuous)
+
+    builder = CurveBuilder(lambda x: "1")
+
+    not_continuous = Vector2(2, 1)
+    continuous = builder.force_continuity(not_continuous)
+    print(continuous)
 
 
 
